# fmt: off
# ruff: noqa
# type: ignore

__codegen__ = True

# Config classes
from nshtrainer.loggers.wandb import BaseLoggerConfig as BaseLoggerConfig
from nshtrainer.loggers.wandb import WandbLoggerConfig as WandbLoggerConfig
from nshtrainer.loggers.wandb import WandbUploadCodeConfig as WandbUploadCodeConfig
from nshtrainer.loggers.wandb import CallbackConfigBase as CallbackConfigBase
from nshtrainer.loggers.wandb import WandbWatchConfig as WandbWatchConfig

# Type aliases

# Submodule exports
