# fmt: off
# ruff: noqa
# type: ignore

__codegen__ = True

# Config classes
from nshtrainer.callbacks.debug_flag import DebugFlagCallbackConfig as DebugFlagCallbackConfig
from nshtrainer.callbacks.debug_flag import CallbackConfigBase as CallbackConfigBase

# Type aliases

# Submodule exports
