# fmt: off
# ruff: noqa
# type: ignore

__codegen__ = True

# Config classes
from nshtrainer.profiler.simple import BaseProfilerConfig as BaseProfilerConfig
from nshtrainer.profiler.simple import SimpleProfilerConfig as SimpleProfilerConfig

# Type aliases

# Submodule exports
