# fmt: off
# ruff: noqa
# type: ignore

__codegen__ = True

# Config classes
from nshtrainer.util.config.duration import EpochsConfig as EpochsConfig
from nshtrainer.util.config.duration import StepsConfig as StepsConfig

# Type aliases
from nshtrainer.util.config.duration import DurationConfig as DurationConfig

# Submodule exports
