# fmt: off
# ruff: noqa
# type: ignore

__codegen__ = True

# Config classes
from nshtrainer.callbacks.checkpoint.on_exception_checkpoint import OnExceptionCheckpointCallbackConfig as OnExceptionCheckpointCallbackConfig
from nshtrainer.callbacks.checkpoint.on_exception_checkpoint import CallbackConfigBase as CallbackConfigBase

# Type aliases

# Submodule exports
