# fmt: off
# ruff: noqa
# type: ignore

__codegen__ = True

# Config classes
from nshtrainer.loggers.tensorboard import TensorboardLoggerConfig as TensorboardLoggerConfig
from nshtrainer.loggers.tensorboard import BaseLoggerConfig as BaseLoggerConfig

# Type aliases

# Submodule exports
