# fmt: off
# ruff: noqa
# type: ignore

__codegen__ = True

# Config classes
from nshtrainer.nn.mlp import MLPConfig as MLPConfig
from nshtrainer.nn.mlp import BaseNonlinearityConfig as BaseNonlinearityConfig

# Type aliases
from nshtrainer.nn.mlp import NonlinearityConfig as NonlinearityConfig

# Submodule exports
