# fmt: off
# ruff: noqa
# type: ignore

__codegen__ = True

# Config classes
from nshtrainer.callbacks.wandb_upload_code import WandbUploadCodeConfig as WandbUploadCodeConfig
from nshtrainer.callbacks.wandb_upload_code import CallbackConfigBase as CallbackConfigBase

# Type aliases

# Submodule exports
